Package: primirTSS
Title: Prediction of pri-miRNA Transcription Start Site
Version: 1.29.0
Author: Pumin Li [aut, cre], Qi Xu [aut], Jie Li [aut], Jin Wang [aut]
Maintainer: Pumin Li <ipumin@163.com>
Description: A fast, convenient tool to identify the TSSs of miRNAs by 
    integrating the data of H3K4me3 and Pol II as well as combining the 
    conservation level and sequence feature, provided within both command-line 
    and graphical interfaces, which achieves a better performance than the 
    previous non-cell-specific methods on miRNA TSSs.
License: GPL-2
Encoding: UTF-8
RoxygenNote: 7.1.0
Depends: R (>= 3.5.0)
Imports: GenomicRanges (>= 1.32.2), S4Vectors (>= 0.18.2), rtracklayer
        (>= 1.40.3), dplyr (>= 0.7.6), stringr (>= 1.3.1), tidyr (>=
        0.8.1), Biostrings (>= 2.48.0), purrr (>= 0.2.5),
        BSgenome.Hsapiens.UCSC.hg38 (>= 1.4.1),
        phastCons100way.UCSC.hg38 (>= 3.7.1), GenomicScores (>= 1.4.1),
        shiny (>= 1.0.5), Gviz (>= 1.24.0), BiocGenerics (>= 0.26.0),
        IRanges (>= 2.14.10), TFBSTools (>= 1.18.0), JASPAR2018 (>=
        1.1.1), tibble (>= 1.4.2), R.utils (>= 2.6.0), stats, utils
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
biocViews: ImmunoOncology, Sequencing, RNASeq, Genetics, Preprocessing,
        Transcription, GeneRegulation
URL: https://github.com/ipumin/primirTSS
BugReports: http://github.com/ipumin/primirTSS/issues
git_url: https://git.bioconductor.org/packages/primirTSS
git_branch: devel
git_last_commit: bbff91f
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-04
NeedsCompilation: no
Packaged: 2025-11-05 00:17:09 UTC; biocbuild
