# Generated by roxygen2: do not edit by hand

export(find_tss)
export(peak_join)
export(peak_merge)
export(plot_primiRNA)
export(run_primirTSSapp)
export(trans_cor)
import(BiocGenerics)
import(GenomicRanges)
import(dplyr, except=c("intersect", "setdiff", "union", "combine"))
importFrom(BSgenome.Hsapiens.UCSC.hg38,Hsapiens)
importFrom(Biostrings,readDNAStringSet)
importFrom(Biostrings,reverseComplement)
importFrom(GenomicScores,gscores)
importFrom(IRanges,IRanges)
importFrom(IRanges,countOverlaps)
importFrom(IRanges,findOverlaps)
importFrom(JASPAR2018,JASPAR2018)
importFrom(R.utils,gunzip)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,subjectHits)
importFrom(phastCons100way.UCSC.hg38,phastCons100way.UCSC.hg38)
importFrom(purrr,pmap)
importFrom(rtracklayer,chrom)
importFrom(rtracklayer,import.chain)
importFrom(rtracklayer,liftOver)
importFrom(stats,runif)
importFrom(stringr,str_detect)
importFrom(stringr,str_remove)
importFrom(stringr,str_split)
importFrom(stringr,str_to_title)
importFrom(tibble,as_tibble)
importFrom(tidyr,separate)
importFrom(tidyr,unite)
importFrom(utils,download.file)
